package iotcloud;

import java.nio.ByteBuffer;

/**
 * This Entry records the abort sent by a given machine.
 * @author Ali Younis <ayounis@uci.edu>
 * @version 1.0
 */


class Abort extends Entry {
	private long seqnum;
	private long machineid;

	Abort(Slot slot, long _seqnum, long _machineid) {
		super(slot);
		seqnum=_seqnum;
		machineid=_machineid;
	}

	long getMachineID() {
		return machineid;
	}

	long getSequenceNumber() {
		return seqnum;
	}

	static Entry decode(Slot slot, ByteBuffer bb) {
		long seqnum=bb.getLong();
		long machineid=bb.getLong();
		return new Abort(slot, seqnum, machineid);
	}

	void encode(ByteBuffer bb) {
		bb.put(Entry.TypeAbort);
		bb.putLong(seqnum);
		bb.putLong(machineid);
	}

	int getSize() {
		return 2*Long.BYTES+Byte.BYTES;
	}

	byte getType() {
		return Entry.TypeAbort;
	}

	Entry getCopy(Slot s) {
		return new Abort(s, seqnum, machineid);
	}
}