package iotcloud;

import java.util.Set;
import java.util.HashSet;
import java.nio.ByteBuffer;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;
import java.lang.NullPointerException;


class Guard {

    static final byte Equal = 1;
    static final byte NotEqual = 2;

    private IoTString booleanExpression;

    public Guard() {
        booleanExpression = null;
    }

    public Guard(IoTString _booleanExpression) {
        booleanExpression = _booleanExpression;
    }

    /**
     * Create an equality expression for a key value.
     *
     */
    public static String createExpression(IoTString keyName, IoTString keyValue, byte op) {
        if (op == Equal) {
            return keyName.toString() + "=='" + keyValue.toString() + "'";
        } else if (op == NotEqual) {
            return keyName.toString() + "!='" + keyValue.toString() + "'";
        }

        // Unrecognized op
        return null;
    }

    /**
     * Add a boolean expression to the guard.
     *
     */
    public void setGuardExpression(String expr) {
        booleanExpression = new IoTString(expr);
    }

    /**
     * Evaluate the guard expression for a given set of key value pairs.
     *
     */
    public boolean evaluate(Set<KeyValue> kvSet) throws ScriptException, NullPointerException {

        // There are no conditions to evaluate
        if (booleanExpression == null) {
            return true;
        }

        // All the current key value pairs that we need to evaluate the condition
        String[] variables = new String[kvSet.size()];

        // Fill the variables array
        int i = 0;
        for (KeyValue kv : kvSet) {
            variables[i] = kv.getKey() + " ='" + kv.getValue() + "'";
            i++;
        }

        // Prep the evaluation engine (script engine)
        ScriptEngine engine = new ScriptEngineManager().getEngineByName("JavaScript");
        for (String s : variables) {
            engine.eval(s);
        }

        // Evaluate the guard condition
        return 1 == (Integer)engine.eval(booleanExpression.toString());
    }

    /**
     * Get the size of the guard condition
     *
     */
    public int getSize() {
        return Integer.BYTES + booleanExpression.length();
    }

    public void encode(ByteBuffer bb) {
        bb.putInt(booleanExpression.length());
        bb.put(booleanExpression.internalBytes());
    }

    static Guard decode(ByteBuffer bb) {
        int exprLength = bb.getInt();
        byte[] expr = new byte[exprLength];
        bb.get(expr);
        return new Guard(IoTString.shallow(expr));
    }
}