package iotcloud;

import java.util.Set;
import java.util.HashSet;

import javax.script.ScriptException;
import java.lang.NullPointerException;


class PendingTransaction {

    static final byte Equal = Guard.Equal;
    static final byte NotEqual = Guard.NotEqual;

    private Set<KeyValue> keyValueUpdateSet;
    private Guard guard;

    public PendingTransaction() {
        keyValueUpdateSet = new HashSet<KeyValue>();
        guard = new Guard();
    }

    /**
     * Add a new key value to the updates
     *
     */
    public void addKV(KeyValue newKV) {

        // Make sure there are no duplicates
        for (KeyValue kv : keyValueUpdateSet) {
            if (kv.getKey().equals(newKV.getKey())) {

                // Remove key if we are adding a newer version of the same key
                keyValueUpdateSet.remove(kv);
                break;
            }
        }

        // Add the key to the hash set
        keyValueUpdateSet.add(newKV);
    }

    /**
     * Get the key value update set
     *
     */
    public Set<KeyValue> getKVUpdates() {
        return keyValueUpdateSet;
    }

    /**
    * Get the guard
    *
    */
    public Guard getGuard() {
        return guard;
    }

    /**
     * Add a guard to this transaction
     *
     */
    public void addGuard(Guard _guard) {
        guard = _guard;
    }

    /**
     * Evaluate the guard expression for a given transaction using a set of key value pairs.
     *
     */
    public boolean evaluate(Set<KeyValue> kvSet) throws ScriptException, NullPointerException {

        // Evaluate the guard using the current KV Set
        return guard.evaluate(kvSet);
    }

    /**
     * Add a boolean expression to the guard.
     *
     */
    public void setGuardExpression(String expr) {
        guard.setGuardExpression(expr);
    }

    /**
     * Trampoline static method.
     *
     */
    public static String createExpression(IoTString keyName, IoTString keyValue, byte op) {
        return Guard.createExpression(keyName, keyValue, op);
    }
}