package iotcloud;

import java.nio.ByteBuffer;
import java.util.Set;
import java.util.HashSet;

class Transaction extends Entry {

    private long seqnum;
    private long machineid;
    private Set<KeyValue> keyValueUpdateSet;
    private Guard guard;

    public Transaction(Slot slot, long _seqnum, long _machineid, Set<KeyValue> _keyValueUpdateSet, Guard _guard) {
        super(slot);
        seqnum = _seqnum;
        machineid = _machineid;
        keyValueUpdateSet = _keyValueUpdateSet;
        guard = _guard;
    }

    public long getMachineID() {
        return machineid;
    }

    public long getSequenceNumber() {
        return seqnum;
    }

    public byte getType() {
        return Entry.TypeLastMessage;
    }

    public int getSize() {
        int size = 2 * Long.BYTES + Byte.BYTES; // seq, machine id, entry type
        size += Integer.BYTES; // number of KV's

        // Size of each KV
        for (KeyValue kv : keyValueUpdateSet) {
            size += kv.getSize();
        }

        // Size of the guard
        size += guard.getSize();

        return size;
    }


    public void encode(ByteBuffer bb) {
        bb.put(Entry.TypeTransaction);
        bb.putLong(seqnum);
        bb.putLong(machineid);

        for (KeyValue kv : keyValueUpdateSet) {
            kv.encode(bb);
        }

        guard.encode(bb);
    }

    static Entry decode(Slot slot, ByteBuffer bb) {
        long seqnum = bb.getLong();
        long machineid = bb.getLong();
        int numberOfKeys = bb.getInt();

        Set<KeyValue> kvSet = new HashSet<KeyValue>();

        for (int i = 0; i < numberOfKeys; i++) {
            KeyValue kv = KeyValue.decode(bb);
            kvSet.add(kv);
        }

        Guard guard = Guard.decode(bb);

        return new Transaction(slot, seqnum, machineid, kvSet, guard);
    }



    public Entry getCopy(Slot s) {
        return new Transaction(s, seqnum, machineid, keyValueUpdateSet, guard);
    }
}